package jp.co.pwv.test;

import javax.servlet.ServletContext;

import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockServletContext;
import org.springframework.test.AbstractTransactionalSpringContextTests;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.RequestContextListener;
import org.springframework.web.context.request.RequestScope;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.context.request.SessionScope;
import org.springframework.web.context.support.GenericWebApplicationContext;

abstract public class AbstractTransactionalSpringWebContextTests extends
		AbstractTransactionalSpringContextTests {
	private static final String REQUEST_ATTRIBUTES_ATTRIBUTE =
		RequestContextListener.class.getName() + ".REQUEST_ATTRIBUTES";

	/**
	 * GenericApplicationContext̑GenericWebApplicatinContextԂ悤ɏCB<br>
	 * @see org.springframework.test.AbstractSingleSpringContextTests#createApplicationContext(java.lang.String[])
	 */
	protected ConfigurableApplicationContext createApplicationContext(final String[] locations) {
		GenericWebApplicationContext context = new GenericWebApplicationContext(new GenericApplicationContext().getDefaultListableBeanFactory());
		context.setServletContext(new MockServletContext());
		customizeBeanFactory(context.getDefaultListableBeanFactory());
		createBeanDefinitionReader(context).loadBeanDefinitions(locations);
		context.refresh();
		// StaticWebApplicationContext postProcessBeanFactoryƓǉ
		ConfigurableListableBeanFactory beanFactory = context.getBeanFactory();
		beanFactory.registerScope(WebApplicationContext.SCOPE_REQUEST, new RequestScope());
		beanFactory.registerScope(WebApplicationContext.SCOPE_SESSION, new SessionScope(false));
		beanFactory.registerScope(WebApplicationContext.SCOPE_GLOBAL_SESSION, new SessionScope(true));

		return context;
	}

	/**
	 * Filteȓrequest̑OɌĂяoB
	 * @param req
	 */
	public void beginRequest(MockHttpServletRequest req) {		
		ServletRequestAttributes attributes = new ServletRequestAttributes(req);
		req.setAttribute(REQUEST_ATTRIBUTES_ATTRIBUTE, attributes);
		RequestContextHolder.setRequestAttributes(attributes);
		
		ServletContext sc = req.getSession().getServletContext();
		GenericWebApplicationContext wac = (GenericWebApplicationContext)getApplicationContext();
		wac.setServletContext(sc);
		sc.setAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE, (GenericWebApplicationContext)getApplicationContext());
		
	}
	/**
	 * FilteȓrequesťɌĂяoB
	 * @param req
	 */
	public void endRequest(MockHttpServletRequest req) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) req
				.getAttribute(REQUEST_ATTRIBUTES_ATTRIBUTE);
		ServletRequestAttributes threadAttributes = (ServletRequestAttributes) RequestContextHolder
				.getRequestAttributes();
		if (threadAttributes != null) {
			// We're assumably within the original request thread...
			if (attributes == null) {
				attributes = threadAttributes;
			}
			RequestContextHolder.setRequestAttributes(null);
			LocaleContextHolder.setLocale(null);
		}
		if (attributes != null) {
			attributes.requestCompleted();
			if (logger.isDebugEnabled()) {
				logger.debug("Cleared thread-bound request context: " + req);
			}
		}
	}

}
